@@ -1,6 +1,7 @@
 Lexis <-
 function(entry, exit, duration, entry.status=0, exit.status=0, id, data,
-         merge=TRUE, states, tol=.Machine$double.eps^0.5)
+         merge=TRUE, states, tol=.Machine$double.eps^0.5,
+         keep.dropped=FALSE )
 {
   nmissing <- missing(entry) + missing(exit) + missing(duration)
   if (nmissing > 2)
@@ -176,10 +177,11 @@ function(entry, exit, duration, entry.status=0, exit.status=0, id, data,
       stop("Duration is not the same on all time scales")
     }
   }
-  ## Check that duration is positive
-  if (any(duration<0)) {
-    stop("Duration must be non-negative")
-  }
+#  Taken care of by the code that detects whether lex.du <= tol
+#  ## Check that duration is positive
+#  if (any(duration<0)) {
+#    stop("Duration must be non-negative")
+#  }
 
   ## Make sure id value - if supplied - is valid. Otherwise supply default id
 
@@ -219,13 +221,18 @@ function(entry, exit, duration, entry.status=0, exit.status=0, id, data,
     lex <- cbind(lex, data)
   }
 
-  ## Drop rows with short duration for consistency with splitLexis
+  ## Drop rows with short or negantive duration for consistency with splitLexis
   short.dur <- lex$lex.dur <= tol
-  if (any(short.dur)) {
+  if ( any(short.dur) ) {
       warning("Dropping ", sum(short.dur),
-              " rows with duration of follow up < tol")
+              " rows with duration of follow up < tol\n",
+      if( keep.dropped ) "  The dropped rows are in the attribute 'dropped'\n",
+      if( keep.dropped ) "  To see them type attr(Obj,'dropped'),\n",
+      if( keep.dropped ) "  to get rid of them type attr(Obj,'dropped') <- NULL\n",
+      if( keep.dropped ) "  - where 'Obj' is the name of your Lexis object" )
       lex <- subset(lex, !short.dur)
-  }
+      if( keep.dropped ) attr(lex,"dropped") <- subset(data, short.dur)
+      }
 
   ## Return Lexis object
   attr(lex,"time.scales") <- all.time.scales
@@ -245,12 +252,12 @@ is.Lexis <- function(x)
 check.time.scale <- function(lex, time.scale=NULL)
 {
 
-  ##Utility function, returns the names of the time scales in a Lexis object
-  ##lex - a Lexis object
-  ##time.scale - a numeric or character vector. The function checks that
-  ##             these are valid time scales for the Lexis object.
-  ##Return value is a character vector containing the  names of the requested
-  ##time scales
+  ## Utility function, returns the names of the time scales in a Lexis object
+  ## lex - a Lexis object
+  ## time.scale - a numeric or character vector. The function checks that
+  ##              these are valid time scales for the Lexis object.
+  ## Return value is a character vector containing the  names of the requested
+  ## time scales
 
   all.names <- timeScales(lex)
   if (is.null(time.scale))
@@ -261,12 +268,12 @@ check.time.scale <- function(lex, time.scale=NULL)
   if (is.character(time.scale)) {
     for (i in 1:nscale) {
       if (is.null(lex[[time.scale[i]]]))
-        stop("invalid time scale name")
+        stop(time.scale[i], " is not a valid time scale name")
     }
   }
   else if (is.numeric(time.scale)) {
     if (any(time.scale > length(all.names))  || any(time.scale < 1))
-      stop("invalid time scale column number")
+      stop(time.scale, " not valid time scale column number(s)")
     time.scale <- all.names[time.scale]
   }
   else {
@@ -512,10 +519,10 @@ function( x, time.scale=options()[["Lexis.time.scale"]], digits=1, ... )
 
 ### Generic functions
 
-### Methods for data.frame drop Lexis attributes, so we need a Lexis
-### method that adds them again
+### Methods for data.frame drop Lexis attributes, so we need Lexis
+### methods that retain them
 
-subset.Lexis <- function(x, ...)
+subset.Lexis <- function(x, ... )
 {
   y <-  subset.data.frame(x, ...)
   attr(y,"breaks") <- attr(x, "breaks")
@@ -524,6 +531,26 @@ subset.Lexis <- function(x, ...)
   return(y)
 }
 
+`[.Lexis` <-
+function(x, ...)
+{
+  y <- NextMethod(x)
+  if (is.data.frame(y)) {
+    for (a in c("class", "time.scales", "time.since", "breaks")) {
+        data.table::setattr(y ,a, attr(x, a))}
+  }
+  y
+}
+
+# `[.Lexis` <-
+# function( x, ... )
+# {
+#     structure( NextMethod(),
+#                breaks = attr(x, "breaks"),
+#           time.scales = attr(x, "time.scales"),
+#           time.since  = attr(x, "time.since") )
+# }
+
 merge.data.frame <- function(x, y, ...)
 {
   if (is.Lexis(x))
@@ -560,6 +587,74 @@ merge.Lexis <- function(x, y, id, by, ...)
   return(z)
 }
 
+cbind.Lexis <-
+function( ... )
+{
+allargs <- list( ... )
+# Check that at least one argument is Lexis
+is.lex <- sapply( allargs, inherits, "Lexis" )
+if( all(!is.lex) ) stop( "At least one argument nust be a Lexis object\n",
+                         "and none of the given are.\n")
+if( sum(is.lex)>1 ) stop( "It is meaningless to 'cbind' several Lexis objects:",
+                          " arguments ", paste( which(is.lex), collapse=","),
+                          " are Lexis objects.\n" )
+is.lex <- which(is.lex) 
+res <- do.call( base::cbind.data.frame, allargs )
+attr( res, "class"       ) <- attr( allargs[[is.lex]], "class"       )
+attr( res, "breaks"      ) <- attr( allargs[[is.lex]], "breaks"      )
+attr( res, "time.scales" ) <- attr( allargs[[is.lex]], "time.scales" )
+attr( res, "time.since"  ) <- attr( allargs[[is.lex]], "time.since"  )
+res
+}
+
+rbind.Lexis <-
+function( ... )
+{
+# A list of all Lexis objects    
+allargs <- list( ... )
+# Check if they are all Lexis
+# (or possibly NULL - often rbind-ing with NULL is very useful)
+is.lex <- sapply( allargs, inherits, "Lexis" )
+is.nul <- sapply( allargs, is.null )
+if( !all(is.lex[!is.nul]) )
+    stop( "All arguments must be Lexis objects,\n",
+          "arguments number ", which(!is.lex & !is.null), " are not." )
+# Put them all together
+allargs <- allargs[!is.nul]
+res <- plyr::rbind.fill( allargs )
+# Get the union of time.scale names and the corresponding time.since
+tscl <- do.call( c, lapply( allargs, function(x) attr(x,"time.scales") ) )
+tsin <- do.call( c, lapply( allargs, function(x) attr(x,"time.since" ) ) )
+# but only one copy of each
+scls <- match( unique(tscl), tscl )
+tscl <- tscl[scls]
+tsin <- tsin[scls]    
+# Fish out the breaks on timescale in turn from all input objects and
+# - if all the non-NULL are identical use this
+# - if not, set the corresponding break to NULL
+newbrks <- list()
+# all the breaks attributes in a list    
+brks <- lapply( allargs, function(x) attr(x,"breaks") )
+# run through the timescales found 
+for( scl in tscl )
+   {
+   # breaks for this timescale in any of the objects   
+   brk <- lapply( brks, function(x) x[[scl]] )
+   # but only the non-null ones
+   brk <- brk[!sapply( brk, is.null )]
+   # if more than one occurrence, all non-NULL breaks should be identical
+   if( ( length(brk)>1 & 
+         all( sapply( brk[-1], function(x) identical(brk[[1]],x) ) ) )
+       | length(brk) == 1 ) newbrks[scl] <- brk[1]
+   else newbrks[scl] <- list(NULL)    
+   }
+# define attributes of the reulting object:
+attr( res, "class"       ) <- c( "Lexis", "data.frame" )
+attr( res, "breaks"      ) <- newbrks
+attr( res, "time.scales" ) <- tscl
+attr( res, "time.since"  ) <- tsin
+res  
+}
 
 ## Extractor functions
 
@@ -610,7 +705,7 @@ status <- function(x, at="exit", by.id = FALSE)
    ave( x[,timeScales(x)[1]], x$lex.id, FUN=if(by.id)
                                              switch(at,
                                                     "entry"=min,
-                                                    "exit"=max)
+                                                     "exit"=max)
                                              else I )
   res <- switch(at, "entry"=x$lex.Cst, "exit"=x$lex.Xst)[wh]
   if( by.id ) names( res ) <- x$lex.id[wh]
@@ -670,9 +765,8 @@ transform.Lexis <- function(`_data`, ... )
     save.at <- attributes(`_data`)
     ## We can't use NextMethod here because of the special scoping rules
     ## used by transform.data.frame
-    y <- base:::transform.data.frame(`_data`, ...)
+    y <- base::transform.data.frame(`_data`, ...)
     save.at[["names"]] <- attr(y, "names")
     attributes(y) <- save.at
     y
 }
-
